@@ -683,12 +683,15 @@ class EurekaServerConf(object):
     def servers_not_in_zone(self):
         return self.__servers_not_in_zone
 
+
 class RegisterException(Exception):
     pass
 
+
 class DiscoverException(Exception):
     pass
 
+
 class EurekaClient(object):
     """
     Example:
@@ -1036,9 +1039,23 @@ class EurekaClient(object):
             return self.__applications
 
     def __try_eureka_server_in_cache(self, fun):
+        ok = False
+        invalid_keys = []
         for z, url in self.__cache_eureka_url.items():
-            _logger.debug("try to do %s in zone[%s] using cached url %s. " % (fun.__name__, z, url))
-            fun(url)
+            try:
+                _logger.debug("Try to do %s in zone[%s] using cached url %s. " % (fun.__name__, z, url))
+                fun(url)
+            except (http_client.HTTPError, http_client.URLError):
+                _logger.warn("Eureka server [%s] is down, use next url to try." % url, exc_info=True)
+                invalid_keys.append(z)
+            else:
+                ok = True
+        if invalid_keys:
+            _logger.debug("Invalid keys::%s will be removed from cache." % str(invalid_keys))
+            for z in invalid_keys:
+                del self.__cache_eureka_url[z]
+        if not ok:
+            raise RegisterException("All eureka servers in cache are down!")
 
     def __try_eureka_server_in_zone(self, fun):
         self.__try_eureka_servers_in_list(fun, self.__eureka_server_conf.servers_in_zone, self.zone)
@@ -1047,27 +1064,27 @@ class EurekaClient(object):
         for zone, urls in self.__eureka_server_conf.servers_not_in_zone.items():
             try:
                 self.__try_eureka_servers_in_list(fun, urls, zone)
-            except (http_client.HTTPError, http_client.URLError):
-                _logger.exception("error!")
+            except RegisterException:
+                _logger.warn("try eureka servers in zone[%s] error!" % zone, exc_info=True)
             else:
                 return
-        raise http_client.URLError("All eureka servers in all zone are down!")
+        raise RegisterException("All eureka servers in all zone are down!")
 
     def __try_eureka_server_regardless_zones(self, fun):
         for zone, urls in self.__eureka_server_conf.servers.items():
             try:
                 self.__try_eureka_servers_in_list(fun, urls, zone)
-            except (http_client.HTTPError, http_client.URLError):
-                _logger.exception("error!")
+            except RegisterException:
+                _logger.warn("try eureka servers in zone[%s] error!" % zone, exc_info=True)
             else:
                 return
-        raise http_client.URLError("All eureka servers in all zone are down!")
+        raise RegisterException("All eureka servers in all zone are down!")
 
     def __try_all_eureka_servers(self, fun):
         if self.__prefer_same_zone:
             try:
                 self.__try_eureka_server_in_zone(fun)
-            except (http_client.HTTPError, http_client.URLError):
+            except RegisterException:
                 self.__try_eureka_server_not_in_zone(fun)
         else:
             self.__try_eureka_server_regardless_zones(fun)
@@ -1082,7 +1099,7 @@ class EurekaClient(object):
                     _logger.debug("try to do %s in zone[%s] using url %s. " % (fun.__name__, _zone, url))
                     fun(url)
                 except (http_client.HTTPError, http_client.URLError):
-                    _logger.exception("Eureka server [%s] is down, use next url to try." % url)
+                    _logger.warn("Eureka server [%s] is down, use next url to try." % url, exc_info=True)
                 else:
                     ok = True
                     self.__cache_eureka_url[_zone] = url
@@ -1091,13 +1108,13 @@ class EurekaClient(object):
             if not ok:
                 if _zone in self.__cache_eureka_url:
                     del self.__cache_eureka_url[_zone]
-                raise http_client.URLError("All eureka servers in zone[%s] are down!" % _zone)
+                raise RegisterException("All eureka servers in zone[%s] are down!" % _zone)
 
     def __connect_to_eureka_server(self, fun):
         if self.__cache_eureka_url:
             try:
                 self.__try_eureka_server_in_cache(fun)
-            except (http_client.HTTPError, http_client.URLError):
+            except RegisterException:
                 self.__try_all_eureka_servers(fun)
         else:
             self.__try_all_eureka_servers(fun)
@@ -1151,7 +1168,7 @@ class EurekaClient(object):
             self.__connect_to_eureka_server(lambda url: _register(url, self.__instance))
         except:
             self.__alive = False
-            _logger.exception("register error!")
+            _logger.warn("Register error! Will try in next heartbeat. ", exc_info=True)
         else:
             _logger.debug("register successfully!")
             self.__alive = True
@@ -1160,7 +1177,7 @@ class EurekaClient(object):
         try:
             self.__connect_to_eureka_server(lambda url: cancel(url, self.__instance["app"], self.__instance["instanceId"]))
         except:
-            _logger.exception("error!")
+            _logger.warn("Cancel error!", exc_info=True)
         else:
             self.__alive = False
 
@@ -1173,8 +1190,7 @@ class EurekaClient(object):
                                                                       self.__instance["instanceId"], self.__instance["lastDirtyTimestamp"],
                                                                       status=self.__instance["status"], overriddenstatus=overridden_status))
         except:
-            _logger.exception("Error!")
-            _logger.info("Cannot send heartbeat to server, try to register")
+            _logger.warn("Cannot send heartbeat to server, try to register. ", exc_info=True)
             self.register()
 
     def status_update(self, new_status):
@@ -1183,7 +1199,7 @@ class EurekaClient(object):
             self.__connect_to_eureka_server(lambda url: status_update(url, self.__instance["app"], self.__instance["instanceId"],
                                                                       self.__instance["lastDirtyTimestamp"], new_status))
         except:
-            _logger.exception("error!")
+            _logger.warn("update status error!", exc_info=True)
 
     def delete_status_override(self):
         self.__connect_to_eureka_server(lambda url: delete_status_override(
@@ -1215,7 +1231,7 @@ class EurekaClient(object):
         try:
             self.__connect_to_eureka_server(do_pull)
         except:
-            _logger.exception("pull full registry from eureka server error!")
+            _logger.warn("pull full registry from eureka server error!", exc_info=True)
 
     def __fetch_delta(self):
         def do_fetch(url):
@@ -1235,7 +1251,7 @@ class EurekaClient(object):
         try:
             self.__connect_to_eureka_server(do_fetch)
         except:
-            _logger.exception("fetch delta from eureka server error!")
+            _logger.warn("fetch delta from eureka server error!", exc_info=True)
 
     def __is_hash_match(self):
         app_hash = self.__get_applications_hash()
